@@ -1,16 +1,20 @@
 #! /usr/bin/perl -w
 use strict;
+$| = 1;
+
+# $Id: synctree.pl 919 2005-12-18 10:06:39Z abeltje $
+use vars qw( $VERSION );
+$VERSION = '0.011';
 
-use Getopt::Long;
 use File::Spec;
 use FindBin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
-use Test::Smoke::Syncer;
-
+use lib $FindBin::Bin;
 use Test::Smoke;
-use vars qw( $VERSION );
-$VERSION = '0.008';
+use Test::Smoke::Syncer;
+use Test::Smoke::Util qw( do_pod2usage );
 
+use Getopt::Long;
 my %opt = (
     type   => undef,
     ddir   => undef,
@@ -23,7 +27,7 @@ my %opt = (
 
 my $defaults = Test::Smoke::Syncer->config( 'all_defaults' );
 
-my %valid_type = map { $_ => 1 } qw( rsync snapshot copy hardlink forest );
+my %valid_type = map { $_ => 1 } qw( rsync snapshot copy hardlink forest ftp );
 
 =head1 NAME
 
@@ -53,27 +57,29 @@ Other options can override the settings from the configuration file.
 =item * B<General options>
 
     -d | --ddir <directory>  Set the directory for the source-tree
-    -v | --verbose           Be verbose
+    -t | --type <type>       'rsync', 'snapshot', 'copy', 'ftp' [mandatory]
 
-    -t | --type <type>       'rsync', 'snapshot', 'copy' [mandatory]
+    -v | --verbose <0..2>    Set verbose level
+    -h | --help              Show help message (needs Pod::Usage)
+    --man                    Show the perldoc  (needs Pod::Usage)
 
 =item * B<options for> -t rsync
 
-    --source <rsync-src>     (ftp.linux.activestate.com::perl-current)
+    --source <rsync-src>     (public.activestate.com::perl-current)
     --rsync <path/to/rsync>  (rsync)
     --opts <rsync-opts>      (-az --delete)
 
 =item * B<options for> -t snapshot
 
-    --server <ftp-server>    (ftp.funet.fi)
-    --sdir <directory>       (/pub/languages/perl/snap)
+    --server <ftp-server>    (public.activestate.com)
+    --sdir <directory>       (/pub/apc/perl-current-snap)
     --sfile <file>           ('')
     --snapext <ext>          (tgz)
     --tar <un-tar-gz>        (gzip -dc %s | tar -xf -)
 
     --patchup                patch a snapshot [needs the patch program]
-    --pserver <ftp-server>   (ftp2.activestate.com)
-    --pdir <directory>       (/pub/staff/gsar/APC/perl-current-diffs)
+    --pserver <ftp-server>   (public.activestate.com)
+    --pdir <directory>       (/pub/apc/perl-current-diffs)
     --unzip <command>        (gzip -dc)
     --patch <command>        (patch)
     --cleanup <level>        (0) none; (1) snapshot; (2) diffs; (3) both
@@ -84,7 +90,13 @@ Other options can override the settings from the configuration file.
 
 =item * B<options for> -t hardlink
 
-    --hdir <directory>       Source directory to hardlink from
+    --hdir <directory>     Source directory to hardlink from
+
+=item * B<options for> -t ftp
+
+    --ftphost              Host with sources (public.activestate.com)
+    --ftpsdir              Sourcedir to mirror (/pub/apc/perl-current)
+    --ftpcdir              Diffs dir to get change (/pub/apc/perl-current-diffs)
 
 =item * B<options for> -t forest
 
@@ -95,10 +107,15 @@ Other options can override the settings from the configuration file.
 
 =back
 
+=head1 DESCRIPTION
+
+This is a small front-end for L<Test::Smoke::Syncer>.
+
 =cut
 
+my $myusage = "Usage: $0 -t rsync -d <destdir>";
 GetOptions( \%opt,
-    'type|t=s', 'ddir|d=s', 'v|verbose:i',
+    'type|t=s', 'ddir|d=s', 'v|verbose=i',
 
     'source=s', 'rsync=s', 'opts',
 
@@ -112,10 +129,10 @@ GetOptions( \%opt,
     'help|h', 'man|m',
 
     'config|c:s',
-) or do_pod2usage( verbose => 1 );
+) or do_pod2usage( verbose => 1, myusage => $myusage );
 
-$opt{man}  and do_pod2usage( verbose => 2, exitval => 0 );
-$opt{help} and do_pod2usage( verbose => 1, exitval => 0 );
+$opt{ man} and do_pod2usage( verbose => 2, exitval => 0, myusage => $myusage );
+$opt{help} and do_pod2usage( verbose => 1, exitval => 0, myusage => $myusage );
 
 if ( defined $opt{config} ) {
     $opt{config} eq "" and $opt{config} = 'smokecurrent_config';
@@ -155,23 +172,6 @@ $patchlevel = $syncer->sync;
 
 $opt{v} and print "$opt{ddir} now up to patchlevel $patchlevel\n";
 
-sub do_pod2usage {
-    eval { require Pod::Usage };
-    if ( $@ ) {
-        print <<EO_MSG;
-Usage: $0 -t <type> -d <directory> [options]
-
-Use 'perldoc $0' for the documentation.
-Please install 'Pod::Usage' for easy access to the docs.
-
-EO_MSG
-        my %p2u_opt = @_;
-        exit( exists $p2u_opt{exitval} ? $p2u_opt{exitval} : 1 );
-    } else {
-        Pod::Usage::pod2usage( @_ );
-    }
-}
-
 =head1 SEE ALSO
 
 L<perlhack/"Keeping in sync">, L<Test::Smoke::Syncer>
